@@ -9,7 +9,7 @@
     if (!inherits(x, "rda"))
         stop("biplot can be used only with linear ordination (e.g., PCA)")
     else
-        NextMethod("biplot", x, ...)
+        NextMethod("biplot")
 }
 
 `biplot.rda` <- function(x, choices = c(1, 2), scaling = "species",
@@ -25,7 +25,7 @@
   if (length(col) == 1)
       col <- c(col,col)
   g <- scores(x, choices = choices, display = display,
-              scaling = scaling, const)
+              scaling = scaling, correlation = correlation, const)
   if (!is.list(g)) {
       g <- list(default = g)
       names(g) <- display
