@@ -7,7 +7,8 @@
     if (is.null(object$terms))
         stop("ordination model must be fitted using formula")
     test <- match.arg(test)
-    out <- NextMethod("drop1", object, test="none", ...)
+    # don't show messages about aliased terms
+    out <- suppressMessages(NextMethod("drop1", object, test = "none"))
     cl <- class(out)
     if (test == "permutation") {
         rn <- rownames(out)[-1]
