@@ -142,7 +142,7 @@
     ## QR decomposition
     Q <- qr(Z)
     if (Q$rank == 0) # nothing partialled out
-        return(list(Y = Y, result = NULL))
+        return(list(Y = Y, result = list(rank = 0, tot.chi = 0)))
     ## partialled out variation as a trace of Yfit
     Yfit <- qr.fitted(Q, Y)
     if (DISTBASED) {
@@ -195,14 +195,32 @@
     Q <- qr(X)
     ## we need to see how much rank grows over rank of conditions
     rank <- sum(Q$pivot[seq_len(Q$rank)] > zcol)
-    ## nothing explained (e.g., constant constrain)
-    if (rank == 0)
-        return(list(Y = Y, result = NULL))
     ## check for aliased terms
-    if (length(Q$pivot) > Q$rank)
-        alias <- colnames(Q$qr)[-seq_len(Q$rank)]
+    if (length(Q$pivot) > Q$rank) {
+        if (Q$rank > 0)
+            alias <- colnames(Q$qr)[-seq_len(Q$rank)] # Q$rank 0, then alias ""
+        else
+            alias <- colnames(Q$qr)
+        # print a message to highlight this aliasing
+        #msg <- "Some model terms were linearly dependent and their effects
+#cannot be uniquely estimated. See '?alias.cca' for more detail and use
+#'vif.cca()' to identify these terms."
+        #message(strwrap(msg, prefix = "\n", initial = "\n",
+        #    width = 0.95 * getOption("width")))
+        aliased <- paste(sQuote(alias), collapse = ", ")
+        msg <- paste("Some constraints or conditions were aliased because they were redundant.",
+        "This can happen if terms are constant or linearly dependent (collinear):", aliased)
+        message(strwrap(msg, width = getOption("width"), prefix = "\n", initial = "\n"))
+    }
     else
         alias <- NULL
+
+    ## nothing explained: constant constrain, complete alias by Conditions...
+    if (rank == 0) {
+        return(list(Y = Y, result = list(rank = 0, tot.chi = 0,
+                                         qrank = 0, alias = alias)))
+    }
+
     ## kept constraints and their means
     kept <- seq_along(Q$pivot) <= Q$rank & Q$pivot > zcol
     if (zcol > 0)
@@ -259,12 +277,14 @@
                            "CAPSCALE" = "CAP",
                            "DISTBASED" = "dbRDA"),
                     seq_len(sum(posev)))
+    if (!any(posev)) # issue #670: no positive eigenvalues in dbrda
+        axnam <- NULL
     if (DISTBASED && any(!posev))
         negnam <- paste0("idbRDA", seq_len(sum(!posev)))
     else
         negnam <- NULL
     dnam <- dimnames(Y)
-    if (any(posev))
+    if (any(posev) || any(!posev))
         names(lambda) <- c(axnam, negnam)
     if (ncol(u))
         dimnames(u) <- list(dnam[[1]], c(axnam, negnam))
@@ -283,8 +303,8 @@
         wa = wa,
         alias = alias,
         biplot = bp,
-        rank = length(lambda),
-        qrank = rank,
+        rank = length(lambda),  # no. of positive eigenvalues
+        qrank = rank,           # rank of constraints after conditions
         tot.chi = sum(lambda),
         QR = Q,
         envcentre = envcentre)
@@ -416,9 +436,11 @@
     }
     ## Residuals
     resid <- ordResid(Y)
-    if (resid$rank < 1)
-        warning("overfitted model with no unconstrained component",
-                call. = FALSE)
+    if (resid$rank < 1) {
+        msg <- "The model is overfitted with no unconstrained (residual) component"
+        message(strwrap(msg, prefix = "\n", initial = "\n",
+            width = 0.95 * getOption("width")))
+    }
     ## return a CCA object
     out <- c(head,
              call = match.call(),
