@@ -72,12 +72,12 @@
         nsimul <- nsimul - colSums(is.na(permutations))
     }
     p <- rep(NA, length(object$statistic))
-    for(i in seq_along(p)) 
+    for(i in seq_along(p))
         p[i] <- switch(alt[i],
-                       two.sided = 2*pmin(pless[i], pmore[i]),
+                       two.sided = 2*pmin.int(pless[i], pmore[i]),
                        greater = pmore[i],
                        less = pless[i])
-    object$p <- pmin(1, (p + 1)/(nsimul + 1))
+    object$p <- pmin.int(1, (p + 1)/(nsimul + 1))
     ## out
     class(object) <- "summary.permustats"
     object
@@ -124,19 +124,38 @@
               class = "permustats")
 }
 
-### densityplot
+### Wrapper function to allow calling S3 method functions densityplot
+### & qqmath without loading (attaching) lattice package.
+
+`permulattice` <-
+    function(x, plot = c("densityplot", "qqmath"), observed = TRUE,
+             axislab = "Permutations", ...)
+{
+    plot <- match.arg(plot)
+    switch(plot,
+           "densityplot" =
+               densityplot(x, observed = observed, xlab = axislab, ...),
+           "qqmath" =
+               qqmath(x, observed = observed, ylab = axislab, ...)
+           )
+}
+
+### lattice::densityplot
 
 `densityplot.permustats` <-
-    function(x, data, xlab = "Permutations", ...)
+    function(x, data, observed = TRUE, xlab = "Permutations", ...)
 {
     obs <- x$statistic
-    sim <- rbind(x$statistic, as.matrix(x$permutations))
+    sim <- as.matrix(x$permutations)
+    if (observed)
+        sim <- rbind(x$statistic, sim)
     nm <- names(obs)[col(sim)]
     densityplot( ~ as.vector(sim) | factor(nm, levels = unique(nm)),
                 xlab = xlab,
                 panel = function(x, ...) {
                     panel.densityplot(x, ...)
-                    panel.abline(v = obs[panel.number()], ...)
+                    if (observed)
+                        panel.abline(v = obs[panel.number()], ...)
                 },
                 ...)
 }
@@ -179,6 +198,8 @@
     invisible(q)
 }
 
+## lattice::qqmath
+
 `qqmath.permustats` <-
     function(x, data, observed = TRUE, sd.scale = FALSE,
              ylab = "Permutations", ...)
@@ -205,6 +226,33 @@
                 ...)
 }
 
+## boxplot for (standardized) effect size: permutations are centred to
+## the value of statistic, and optionally standardized to equal sd
+## w.r.t. to column mean (not to the statistics). This shows the
+## effect size, or the deviation from the observed statistic (= 0).
+
+`boxplot.permustats` <-
+    function(x, scale = FALSE, names, ...)
+{
+    p <- x$permutations
+    if (isTRUE(scale))
+        scale <- apply(p, 2, sd, na.rm = TRUE)
+    p <- scale(p, center = x$statistic, scale = scale)
+    if (missing(names))
+        names <- attr(x$statistic, "names")
+    boxplot(p, names = names, ...)
+}
+
+## pairs plot permuted variables against each other
+
+`pairs.permustats` <-
+    function(x, ...)
+{
+    p <- x$permutations
+    colnames(p) <- attr(x$statistic, "names")
+    pairs(p, ...)
+}
+
 ###
 ### specific methods to extract permustats
 ###
@@ -219,17 +267,7 @@
               class = "permustats")
 }
 
-`permustats.adonis` <-
-    function(x, ...)
-{
-    tab <- x$aov.tab
-    k <- !is.na(tab$F.Model)
-    structure(list(
-        "statistic" = structure(tab$F.Model[k], names = rownames(tab)[k]),
-        "permutations" = x$f.perms,
-        "alternative" = "greater"),
-              class = "permustats")
-}
+## summary: see file R/summary.anosim.R
 
 `permustats.mantel` <-
     function(x, ...)
@@ -241,6 +279,12 @@
               class = "permustats")
 }
 
+`summary.mantel` <-
+    function(object, ...)
+{
+    summary(permustats(object, ...))
+}
+
 `permustats.mrpp` <-
     function(x, ...)
 {
@@ -251,6 +295,12 @@
               class = "permustats")
 }
 
+`summary.mrpp` <-
+    function(object, ...)
+{
+    summary(permustats(object, ...))
+}
+
 `permustats.oecosimu` <-
     function(x, ...)
 {
@@ -261,6 +311,12 @@
               class = "permustats")
 }
 
+`summary.oecosimu` <-
+    function(object, ...)
+{
+    summary(permustats(object, ...))
+}
+
 `permustats.ordiareatest` <-
     function(x, ...)
 {
@@ -271,16 +327,24 @@
               class = "permustats")
 }
 
+`summary.ordiareatest` <-
+    function(object, ...)
+{
+    summary(permustats(object, ...))
+}
+
 `permustats.permutest.cca` <-
     function(x, ...)
 {
     structure(list(
-        "statistic" = structure(x$F.0, names = "F"),
+        "statistic" = structure(x$F.0, names = x$termlabels),
         "permutations" = x$F.perm,
         "alternative" = "greater"),
               class = "permustats")
 }
 
+## no summary: test is a summary by itself
+
 `permustats.protest` <-
     function(x, ...)
 {
@@ -291,6 +355,8 @@
               class = "permustats")
 }
 
+## uses summary.procrustes
+
 ### the following do not return permutation data
 `permustats.CCorA` <-
     function(x, ...)
@@ -337,6 +403,8 @@
               class = "permustats")
 }
 
+## no summary: print is sufficient
+
 `permustats.anova.cca` <-
     function(x, ...)
 {
@@ -351,3 +419,5 @@
        "alternative" = "greater"),
        class = "permustats")
 }
+
+## no summary: anova is a summary by itself
