@@ -7,16 +7,16 @@ permutest.default <- function(x, ...)
 `permutest.cca` <-
     function (x, permutations = how(nperm=99),
               model = c("reduced", "direct", "full"), by = NULL, first = FALSE,
-              strata = NULL, parallel = getOption("mc.cores") ,  ...)
+              strata = NULL, parallel = getOption("mc.cores"), ...)
 {
     ## do something sensible with insensible input (no constraints)
-    if (is.null(x$CCA)) {
+    if (is.null(x$CCA) || x$CCA$rank == 0) {
         sol <- list(call = match.call(), testcall = x$call, model = NA,
-                    F.0 = NA, F.perm = NA, chi = c(0, x$CA$tot.chi),
+                    F.0 = NA, F.perm = NA, chi = c(0, max(x$CA$tot.chi,0)),
                     num = 0, den = x$CA$tot.chi,
-                    df = c(0, nrow(x$CA$u) - max(x$pCCA$QR$rank,0) - 1),
+                    df = c(0, nobs(x) - max(x$pCCA$QR$rank,0) - 1),
                     nperm = 0, method = x$method, first = FALSE,
-                    Random.seed = NA)
+                    termlabels = "<none>", Random.seed = NA)
         class(sol) <- "permutest.cca"
         return(sol)
     }
@@ -31,29 +31,51 @@ permutest.default <- function(x, ...)
     model <- match.arg(model)
     ## special cases
     isCCA <- !inherits(x, "rda")    # weighting
-    isPartial <- !is.null(x$pCCA)   # handle conditions
+    if (isCCA)
+        w <- x$rowsum # works with any na.action, weights(x) won't
+    else
+        w <- NULL
+    isPartial <- !is.null(x$pCCA) && x$pCCA$rank > 0   # handle conditions
     isDB <- inherits(x, c("dbrda")) # only dbrda is distance-based
     ## C function to get the statististics in one loop
-    getF <- function(indx, ...)
+    getF <- function(indx, E, Q, QZ, effects, w, first, isPartial, isCCA,
+                     isDB, q, r)
     {
+        # q is the rank(s) of the effect(s) - 1
+        # r is the rank of the residual term - 1
         if (!is.matrix(indx))
-            indx <- matrix(indx, nrow=1)
-        out <- .Call(do_getF, indx, E, Q, QZ, effects, first, isPartial, isDB)
+            indx <- matrix(indx, nrow = 1)
+        out <- .Call(do_getF, indx, E, Q, QZ, effects, w, first, isPartial,
+                     isCCA, isDB)
         p <- length(effects)
         if (!isPartial && !first)
-            out[,p+1] <- Chi.tot - rowSums(out[,seq_len(p), drop=FALSE])
+            out[, p + 1] <- Chi.tot - rowSums(out[,seq_len(p), drop=FALSE])
         if (p > 1) {
             if (by == "terms")
                 out[, seq_len(p)] <- sweep(out[, seq_len(p), drop = FALSE],
-                                               2, q, "/")
+                                           2, q, "/")
             out <- cbind(out, sweep(out[,seq_len(p), drop=FALSE], 1,
-                                    out[,p+1]/r, "/"))
+                                    out[, p + 1] / r, "/"))
         }
         else
-            out <- cbind(out, (out[,1]/q)/(out[,2]/r))
+            out <- cbind(out, (out[, 1] / q) / (out[, 2] / r))
         out
     }
     ## end getF
+    ## wrapper to getF for in parallel lapply
+    getFlapply <- function(i, permutations, E, Q, QZ, effects, w, first,
+                            isPartial, isCCA, isDB, q, r) {
+        getF(permutations[i,], E = E, Q = Q, QZ = QZ, effects = effects, w = w,
+             first = first, isPartial = isPartial, isCCA = isCCA, isDB = isDB,
+             q = q, r = r)
+    }
+    ## wrapper to getF for parRapply
+    getFparRapply <- function(i, permutations, E, Q, QZ, effects, w, first,
+                              isPartial, isCCA, isDB, q, r) {
+        getF(permutations[i,], E = E, Q = Q, QZ = QZ, effects = effects, w = w,
+             first = first, isPartial = isPartial, isCCA = isCCA, isDB = isDB,
+             q = q, r = r)
+    }
     ## QR decomposition
         Q <- x$CCA$QR
     if (isPartial) {
@@ -148,21 +170,34 @@ permutest.default <- function(x, ...)
     if (hasClus || parallel > 1) {
         if(.Platform$OS.type == "unix" && !hasClus) {
             tmp <- do.call(rbind,
-                           mclapply(1:nperm,
-                                    function(i) getF(permutations[i,]),
-                                    mc.cores = parallel))
+                           mclapply(splitIndices(nperm, parallel),
+                                    getFlapply,
+                                    mc.cores = parallel,
+                                    permutations = permutations,
+                                    E = E, Q = Q, QZ = QZ,
+                                    effects = effects, w = w,
+                                    first = first, isPartial = isPartial,
+                                    isCCA = isCCA, isDB = isDB, q = q, r = r))
         } else {
             ## if hasClus, do not set up and stop a temporary cluster
             if (!hasClus) {
                 parallel <- makeCluster(parallel)
             }
-            tmp <- parRapply(parallel, permutations, function(i) getF(i))
-            tmp <- matrix(tmp, ncol=3, byrow=TRUE)
+            tmp <- do.call(rbind,
+                           parLapply(parallel,
+                                     splitIndices(nperm, length(parallel)),
+                                     getFparRapply,
+                                     permutations=permutations,
+                                     E = E, Q = Q, QZ = QZ, effects = effects, w = w,
+                                     first = first, isPartial = isPartial,
+                                     isCCA = isCCA, isDB = isDB, q = q, r = r))
             if (!hasClus)
                 stopCluster(parallel)
         }
     } else {
-        tmp <- getF(permutations)
+        tmp <- getF(permutations, E = E, Q = Q, QZ = QZ, effects = effects,
+                    w = w, first = first, isPartial = isPartial, isCCA = isCCA,
+                    isDB = isDB, q = q, r = r)
     }
     if ((p <- length(effects)) > 1) {
         num <- tmp[,seq_len(p)]
