@@ -1,4 +1,4 @@
-"text.decorana" <-
+`text.decorana` <-
     function (x, display = c("sites", "species"), labels, choices = 1:2,
               origin = TRUE, select, ...)
 {
@@ -7,10 +7,10 @@
     display <- match.arg(display)
     x <- scores(x, display = display, choices = choices, origin = origin,
                 ...)
+    if (!missing(select))
+        x <- .checkSelect(select, x)
     if (!missing(labels))
         rownames(x) <- labels
-    if (!missing(select))
-        x <- x[select, , drop = FALSE]
     localText(x, rownames(x), ...)
     invisible()
 }
