@@ -1,12 +1,14 @@
 `vif.cca` <-
-    function(object) 
+    function(object)
 {
+    if (is.null(object$CCA) || object$CCA$rank == 0)
+        stop("can be used only with constrained ordination")
     Q <- object$CCA$QR
     out <- rep(NA, NCOL(Q$qr))
     names(out)[Q$pivot] <- colnames(Q$qr)
     rank <- Q$rank
     V <- chol2inv(Q$qr, size = rank)
-    X <- qr.X(Q)[, Q$pivot[1:rank], drop=FALSE]
+    X <- qr.X(Q, ncol = length(Q$pivot))[, Q$pivot[1:rank], drop=FALSE]
     Vi <- crossprod(X)
     v1 <- diag(V)
     v2 <- diag(Vi)
